/**
 * $Id: jAppli.java,v 1.14 2001/06/12 15:50:30 gchomat Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * The FluxMdia Software License, Version 1.0
 *
 *
 * Copyright (c) 2001 FluxMdia.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        FluxMdia (http://www.FluxMdia.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Transmorpher" and "FluxMdia" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact FluxMdia@FluxMdia.org
 *
 * 5. Products derived from this software may not be called "FluxMdia",
 *    nor may "FluxMdia" appear in their name, without prior written
 *    permission of FluxMdia.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL FLUXMEDIA OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of FluxMdia. For more
 * information on FluxMdia, please see <http://www.FluxMdia.org/>.
 */

/**
 * This is the main class of the applicaton. It run the Process factory
 * and parse the xml file given in parameter.
 * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
 * @since SAX 2.0 / jdk 1.2 / XALAN 2.0
 */

package fr.fluxmedia.transmorpher.Application ;

import fr.fluxmedia.transmorpher.Parser.* ;
import fr.fluxmedia.transmorpher.TProcessFactory.* ;

public final class jAppli {


  /**
    *
    *
    *
    */
  public static final void printHelp(CommandLineArgument arg) {
    arg.setBadCommandLineArgument();
    System.out.println("[jAppli] You must give a ProcessFile in argument");
    System.out.println("Usage : java [Javaoption] fr.fluxmedia.transmorpher.Application.jAppli args");
    System.out.println("");
    System.out.println("Where args include :");
    System.out.println("  -p XmlFile    the file which describe the process to execute");
    System.out.println("  -v            print the version number");
    System.out.println("  -h -?         print this help");
  }


/**
* Analyse the command line arguments
*
*/

public static final CommandLineArgument argsAnalyse( String[] pArgs ) {
    int vI = 0;
    CommandLineArgument arg = new CommandLineArgument();

    if ( pArgs.length == 0) {
      // Nothing
      printHelp(arg);
    } else {  // args.length > 0
      while ( vI < pArgs.length ) {
        if ( pArgs[ vI ].equals( "-p" ) ) {
          if ( vI + 1 < pArgs.length ) arg.setFilename(pArgs[ ++vI ]);
          else {
            System.out.println("Missing parameter for filename");
            printHelp(arg);
          }
        } else if ( pArgs[ vI ].equals( "-?" ) || pArgs[ vI ].equals( "-h" )) {
          printHelp(arg);
        } else {
          printHelp(arg);
        }
        vI ++;
      } //while
    } // args.length > 0
    return arg ;
  }

  /**
   * Process a call to a function.
   * This is the main function of the application.
   *
   * @ param the name of the XML file which describe the
             execution process
   */

  public static final void main(String argc[]) {
    //System definition
    System.setErr(System.out);

    // to compute the execution time
    double begin = System.currentTimeMillis();

    CommandLineArgument arg = argsAnalyse(argc);

    if (arg.IsOk()) {
      // build a process factory
      TProcessFactory XSLTPF = null;
      try {
        XSLTPF = new TProcessFactory();
       // build the process parser. Analyse the xml file
       // build the Process describe in the file
        FMParser iProcessParser = new ProcessParser(XSLTPF);  
	iProcessParser.parse(arg.getFilename());
        XSLTPF.execProcess();
        // Run the process
        //TODO : run the XSLTPF
      } catch (Exception e) {
        System.out.println("jAppli : Erreur ["+ e +"]");
      }

      double end = System.currentTimeMillis() ;
      System.out.println("Duration =" + (end-begin));      
    } //else bad command line

  } //end proc

} //end class
