/**
 * $Id: ProcessParser.java,v 1.28 2001/06/13 11:33:56 jerome Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
Author  : Laurent Tardif
Date    : 05 01 2001
Content : Define a parser for KaomiDocument
*/

package fr.fluxmedia.transmorpher.Parser ;


// Imported XML CLasses

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

//Imported SAX Classes

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

//Imported JAVA CLasses

import java.io.IOException;

import java.util.Stack;
import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.Integer;

//Imported TRansmorpher CLASSES

import fr.fluxmedia.transmorpher.Files.* ;
import fr.fluxmedia.transmorpher.TProcessFactory.XML_Port ;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory ;
import fr.fluxmedia.transmorpher.TProcessFactory.* ;

/**
Author  : Laurent Tardif
Date    : 05 01 2001
Content : Define a parse for process description file.
          This parser us a sub parser for ruleset.

*/
public final class ProcessParser
    extends FMParser {
    
    /** the TProcessFactory */
    TProcessFactory iXSLT_PF = null;
    
    /** stack of XLSTProcess Composite*/
    //Stack iContext = new Stack();
    
    /** The current TagElement modified*/
    String iModTagName = null;
    
    /** The reader which start the transformation process */
    //  TProcessBasic_Reader iReader = null;
    
    /** The root for all tempalte */
    TProcessComposite vTemplateList = null ;
    
    /** The iXslFile */
    ReadWrite iXslFile = null;
    
    /** The servlet file */
    ReadWrite servletFile = null;
  
    /** Are we copying some XSLT */
    boolean  inLitteralXSLT = false;
    
    Vector iRemAttList = new Vector(5,5);
    
    /*-------------------------------------------- CONSTRUCTOR  -----------------------------*/
    
    /** The constructor, build the XML PArser but not parse the document */
    public ProcessParser(TProcessFactory pXSLT_PF) {
	
	super();
	iXSLT_PF = pXSLT_PF;
	try {
	    vTemplateList = new TProcessComposite(new String[0], new String[0]);
	    vTemplateList.setName("Template");
	} catch (Exception E) {
	    System.out.println("[ProcessParser] : Error durring the creation of the parser : " +E);
	} //fin try
    }
    /*--------------------------------------------- END CONSTRUCTOR -------------------------*/
    
    public final String listAttribute(Attributes atts){
	String listAttribute="";
	if (atts.getLength()!=0)
	    {
		for(int i=0 ; i<atts.getLength() ; i++){
		    listAttribute=listAttribute+atts.getLocalName(i)+"=\""+atts.getValue(i)+"\" ";
		}
		
	    }
	
	    return listAttribute;
    }
    

    
    
    
    /* ************* */
    /* START ELEMENT */
    /* ************* */
    
    /** Call by the XML parser at the begining of an element */
    public final void startElement(String namespaceURI, String localName, String qname, Attributes atts) {
	try {
	    if ( inLitteralXSLT == true ){
		if(atts!=null) System.out.println(atts.getValue("match"));
		iXslFile.Writeln(2,"<"+qname+" "+listAttribute(atts)+">"); 
		//do not forget all the attributes 
	    } else 
		if (namespaceURI.equals("http://transmorpher.fluxmedia.fr/1.0"))
		    {
			if (localName.equals("transmorpher")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessList(atts);
			} else if (localName.equals("process")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startComplexProcess(atts);
			} else if (localName.equals("main")) {
			    //On appel le run a la fin du main
			    //System.out.println("startElement ProcessParser : " + localName);
			    startComplexProcess(atts); 
			} else if (localName.equals("servlet")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startServlet(atts);
			} else if (localName.equals("generate")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessReader(atts);
			} else if (localName.equals("repeat")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessLoop(atts);
			} else if (localName.equals("dispatch")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessDispatcher(atts);
			} else if (localName.equals("apply-external")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessXslt(atts);
			} else if (localName.equals("merge")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessConnector(atts);
			} else if (localName.equals("serialize")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessSerializer(atts);
			} else if (localName.equals("apply-process")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessRefComplex(atts);
			} else if (localName.equals("query")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    iXslFile = QueryParser.startDefineQuery(atts);
			} else if (localName.equals("apply-query")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    startProcessQuery(atts);
			} else if (localName.equals("ruleset")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    iXslFile=RuleSetParser.startDefineRuleSet(atts);
			} else if (localName.equals("apply-ruleset")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    startRuleSet(atts);
			} else if (localName.equals("select")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    QueryParser.startSelect(iXslFile,atts);
			} else if (localName.equals("maptag")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.startMapTag(iXslFile,atts);
			} else if (localName.equals("addtag")) {
			    System.out.println("startElement ProcessParser :  " + localName+"  Not yet implemented");
			    //RuleSetParser.startAddTag(iXslFile,atts);
			} else if (localName.equals("resubst")) {
			    //System.out.println("startElement ProcessParser :  " + localName+"  Not yet implemented");
			    RuleSetParser.startResubst(iXslFile,atts);
			} else if (localName.equals("extends")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.extend(iXslFile,atts);
			} else if (localName.equals("remtag")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.startRemTag(iXslFile,atts);
			} else if (localName.equals("flatten")) {
			    //System.out.println("startElement ProcessParser : " + localName);
			    RuleSetParser.startFlatten(iXslFile,atts);
			} else if (localName.equals("modtag")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    iModTagName=RuleSetParser.startModTag(iXslFile,atts);
			} else if (localName.equals("mapatt")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.startMapAtt(iRemAttList,iModTagName,iXslFile,atts);
			} else if (localName.equals("rematt")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.startRemAtt(iRemAttList,iModTagName,iXslFile,atts);
			} else if (localName.equals("addatt")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    RuleSetParser.startAddAtt(iXslFile,atts); 
			} else if (localName.equals("param")) {
			    //System.out.println("startElement ProcessParser :  " + localName);
			    QueryParser.startParam(iXslFile,atts);
			} else {
			    System.out.println("startElement ProcessParser : ELEMENT INCONU" + localName);
			} //end if
			
		    }
	    
		else if (namespaceURI.equals("http://www.w3.org/1999/XSL/Transform")) {
		    inLitteralXSLT = true;
		    
		    if(atts!=null) System.out.println(atts.getValue("match"));
		    
		    iXslFile.Writeln(2,"<"+qname+" "+listAttribute(atts)+" >"); 
		    
		    //do not forget all the attributes
 
		} else
		    {
			System.out.println("[ProcessParser]incorect namespacs: " + namespaceURI);
			System.out.println("fullname: " + qname );
			System.out.println("for element :" +localName);
			System.out.println();
		    }
	    
	} catch (TransformerException TE) {
	    System.out.println("[ProcessParser]TransformerException " +TE);
	} catch (SAXException SE) {
	    System.out.println("[ProcessParser]Erreur SAX(" + SE+")");
	    System.out.println("[ProcessParser]Erreur SAX(" + SE.getMessage()+")");
	    System.out.println("[ProcessParser]Erreur SAX(" + SE.getException()+")");
	} catch (IOException IO) {
	System.out.println("IOException" +IO);
	} //end try
	
	
  } // startElement(String namespaceURI, String localName, String qname, Attributes atts)
    

  /* The content of the tags */
  /* No content in Transmorpher so far but in XSLT code */
  public void characters(char ch[], int start, int length) {
    String content = "";
    int end = start + length;
    if( inLitteralXSLT == true ){
	for( int i = start; i < end ; i++ ) content = content + ch[i];
	iXslFile.Writeln(2,content);
    }
  }

  /* *********** */
  /* END ELEMENT */
  /* *********** */

  /** Call by the XML parser at the end of an element */
  public final void endElement(String namespaceURI,String pName, String qName ) {
    try {
	
	if( inLitteralXSLT == true ){
         iXslFile.Writeln(2,"</"+qName+">");
	 if( namespaceURI.equals("http://www.w3.org/1999/XSL/Transform") && pName.equals("template") )
	     inLitteralXSLT = false;
	} else 
	if(namespaceURI.equals("http://transmorpher.fluxmedia.fr/1.0"))
	{

	    if (pName.equals("transmorpher")) {
		endProcessList();
	    } else if (pName.equals("process")) {
		endComplexProcess();
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("main")) {
		endComplexProcess();
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("generate")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("dispatch")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("merge")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("apply-external")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("connector")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("loop")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("serialize")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("apply-process")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("apply-query")) {
		//System.out.println("endElement ProcessParser : " + pName);
	    } else if (pName.equals("query")) {
		//System.out.println("endElement ProcessParser : " + pName);
		QueryParser.endDefineQuery(iXslFile);
	    } else if (pName.equals("select")) {
		//System.out.println("endElement ProcessParser : " + pName);
		QueryParser.endSelect(iXslFile); 
	    } else if (pName.equals("param")) {
		//System.out.println("endElement ProcessParser : " + pName);
		QueryParser.endParam(iXslFile);
	    } else if (pName.equals("ruleset")) {
		//System.out.println("endElement  defineruleset");
		RuleSetParser.endDefineRuleSet(iXslFile);
	    } else if (pName.equals("apply-ruleset")) {
		//System.out.println("endElement  defineruleset");
	    } else if (pName.equals("modtag")) {
		//System.out.println("endElement modtag");
		RuleSetParser.endModTag(iRemAttList,iXslFile);
	    } else if (pName.equals("maptag")) {
		//System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("remtag")) {
		//System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("flatten")) {
		System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("mapatt")) {
		System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("rematt")) {
		//System.out.println("endElement Not imimplemented" +pName);
      	    } else if (pName.equals("addatt")) {
		System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("addtag")) {
		System.out.println("endElement Not imimplemented" +pName);
	    } else if (pName.equals("resubst")) {
		//System.out.println("endElement Not imimplemented" +pName);
		RuleSetParser.endResubst(iXslFile);
	    } else if (pName.equals("extends")) {
		//System.out.println("endElement Not imimplemented" +pName);
		RuleSetParser.endExtend(iXslFile);
	    } else {
		System.out.println("endElement ProcessParser : ELEMENT INCONU" + pName);
	    } //end if

	}
	   else System.out.println("[ProcessParser] endElement : Nom de domaine incorect !!");
	} catch (SAXException SE) {
	    System.out.println("SAXException" +SE);
	} catch (IOException IO) {
	    System.out.println("IOException" +IO);
	} //end try
	   } //end endeLement




  /** start the parsing of a process list */
  protected final void startProcessList(Attributes attrs)
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException {

    TProcess vCurrent = TProcessFactory.newComposite(new String[0],new String[0]);
    vCurrent.setName("Header");
    iContext.push(vCurrent);
  } //end proc


    /** start the creation of a servlet*/
    protected final  void startServlet(Attributes attrs)
	throws TransformerException,IOException,SAXException
    {
    
	String pName = "Composite ";

	ParserAnswer iPA= AnalyseArg(attrs);
	
	if (iPA.getName()==null) iPA.setName(pName);
	
	//    System.out.println("[ProcessParser]Creation d'un complexe : 0.0");
	TProcess vCurrent = TProcessFactory.newComposite(iPA.getIn(),iPA.getOut());
	
	CreateComposite((TProcessComposite)vCurrent,iPA);
	iContext.push(iPA);
	iContext.push(vCurrent);

    } //end proc





  /** start the parsing of a ComplexProcess*/
  protected final  void startComplexProcess(Attributes attrs)
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException 
    {
    
	String pName = "Composite ";

	ParserAnswer iPA= AnalyseArg(attrs);
	
	if (iPA.getName()==null) iPA.setName(pName);
	
	TProcess vCurrent = TProcessFactory.newComposite(iPA.getIn(),iPA.getOut());
	
	CreateComposite((TProcessComposite)vCurrent,iPA);
	iContext.push(iPA);
	iContext.push(vCurrent);

    } //end proc


  /**
  Nbin  : 0
  NbOut : 0
  start the parsing of a ProcessReader
  */
    protected  final void startProcessReader(Attributes attrs)
    throws SAXException,
           IOException {
    String pId ="Reader";
    //System.out.println("analyse arg  d'un reader :");
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("Creation d'un reader :");
    TProcess vCurrent =  iXSLT_PF.newGenerator("xml-reader",iPA.getOut(),iPA.getFilename());
    Create(vCurrent,iPA);
  } //end proc

  /**
  Nbin  : X
  NbOut : Y
  start the parsing of a ProcessReader
  */
  protected  final void startProcessLoop(Attributes attrs)
  throws TransformerException,
         TransformerConfigurationException,
         SAXException,
         IOException {
          
    String pId ="Loop";
   
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
   
    TProcess vCurrent =  iXSLT_PF.newLoop(iPA.getIn(),iPA.getOut(),iPA.getRefOn(),iPA.getCount(),vTemplateList);
    Create(vCurrent,iPA);
  } //end proc


  /**
  NbIn = 0
  NbOut = depend on document
  start the parsing of a ProcessDispatcher
  */

  protected  final void startProcessDispatcher(Attributes attrs) {
    String pId ="Dispatcher";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("Creation d'un Dispatcher :");
    TProcess vCurrent = TProcessFactory.newDispatcher("basic",iPA.getIn(),iPA.getOut());
    Create(vCurrent,iPA);
  } //end proc

  /** start the parsing of a ProcessXslt*/
  protected  final void startProcessXslt(Attributes attrs)
     throws TransformerException,
     TransformerConfigurationException,
     SAXException,
     IOException {

    String pId ="Xslt";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("Creation d'un Xslt :");
    TProcess vCurrent = TProcessFactory.newTransformer("XSLT",iPA.getIn(),iPA.getOut(),iPA.getFilename());
    Create(vCurrent,iPA);

  } //end proc

  /**
  start the parsing of a ProcessReader  */
    protected  final void startProcessQuery(Attributes attrs) throws TransformerException,
	TransformerConfigurationException,
	SAXException,
	IOException  {
	
	TProcess vCurrent =null ;
	String pId ="Query";
	
	ParserAnswer iPA= AnalyseArg(attrs);
	if (iPA.getId()==null) iPA.setId(pId);
	
	//System.out.println(iPA.getRefOn());

	if( iPA.getType().equals("tmq") ){
	if( iPA.getFilename()== null )
	    {
		vCurrent = TProcessFactory.newQuery("tmqin",iPA.getIn(),iPA.getOut(),iPA.getRefOn()+".xsl");
	    } 
	else 
	    {
		vCurrent = TProcessFactory.newQuery("tmqfile",iPA.getIn(),iPA.getOut(),iPA.getFilename());
	    }}
	Create(vCurrent,iPA);
	
    } //end proc

  /** start the parsing of a RuleSet (use of)*/
  protected  final void startRuleSet(Attributes attrs)
     throws TransformerException,
     TransformerConfigurationException,
     SAXException,
     IOException {

    String pId ="RuleSet";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    TProcess vCurrent = TProcessFactory.newRuleSet(iPA.getIn(),iPA.getOut(),iPA.getRefOn());
    Create(vCurrent,iPA);

  } //end proc

  /** start the parsing of a ProcessConnector*/
  protected  final void startProcessConnector(Attributes attrs) {
    String pId ="Connector";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("Creation d'un Connector :");
    TProcess vCurrent = TProcessFactory.newConnector("basic",iPA.getIn(),iPA.getOut());
    Create(vCurrent,iPA);
  } //end proc

   /** start the parsing of a ProcessSerializer*/
   protected  final void startProcessSerializer(Attributes attrs)
    throws SAXException,
    IOException  {

    String pId ="Connector";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("Creation d'un Connector :");
    TProcess vCurrent = TProcessFactory.newSerializer("basic",iPA.getIn(),iPA.getFilename());
    Create(vCurrent,iPA);
  } //end proc

   /** start the parsing of a ProcessSerializer*/
   protected  final void startProcessRefComplex(Attributes attrs)
    throws SAXException,
    IOException  {

    String pId ="Ref Complexe";
    ParserAnswer iPA= AnalyseArg(attrs);
    if (iPA.getId()==null) iPA.setId(pId);
    //System.out.println("[ProcessParser]Creation d'un ref complex:");
    CreateRefComplex(iPA);
  } //end proc


  /** end the parsing of a ComplexProcess*/
  protected  final void endComplexProcess()
   throws SAXException, IOException {

    //System.out.println("begin end complex process");

    TProcessComposite vCurrent = (TProcessComposite) iContext.pop();
    ParserAnswer iPA= (ParserAnswer) iContext.pop();
    vCurrent.CreateLink();
    vCurrent.LinkCompositePort();

    //new fr.fluxmedia.transmorpher.Interface.DisplayingProcess(vFather);
    //System.out.println("fin Coucou" + vFather );
  }

  /** start the parsing of a ProcessList*/
  protected  final void endProcessList() throws SAXException, IOException {
    TProcessComposite vFather = (TProcessComposite) iContext.pop();
/*
    if (iReader !=null) {
      //System.out.println("Start process");
      iReader.execProcess();
    }
*/
  }

  /** Analyze the port Name */
  public static final String[] AnalysePortName(String pListe) {
    StringTokenizer vTokens = new StringTokenizer( pListe, " ", true );
    String[] vAns = new String[vTokens.countTokens()/2+1];
    String vToken = null;
    int i = 0;
    while ( vTokens.hasMoreTokens() ) {
       vToken = vTokens.nextToken();
       if (vToken.equals(" ")) {
       } else {
          vAns[i++]=vToken;
       } //end if
    }//end while
    return vAns;
  } //end proc


  /** analyse the argument of processElement */
  protected final ParserAnswer AnalyseArg(Attributes attrs) {
    ParserAnswer vAns= new ParserAnswer();
    String vAttName = null;
    if (attrs != null) {
      int len = attrs.getLength();
      for (int i = 0; i < len; i++) {
        vAttName= attrs.getQName(i); //.toLowerCase();
        if (vAttName.equals("name")){
          vAns.setName(attrs.getValue(i));
        } else if (vAttName.equals("out")){
           vAns.setOut(AnalysePortName(attrs.getValue(i)));
        } else if (vAttName.equals("in")){
           vAns.setIn(AnalysePortName(attrs.getValue(i)));
        } else if (vAttName.equals("id")){
           vAns.setId(attrs.getValue(i));
       }  else if (vAttName.equals("file")){
           vAns.setFilename(attrs.getValue(i));
        } else if (vAttName.equals("type")){
           vAns.setType(attrs.getValue(i));
        } else if (vAttName.equals("ref")){
           vAns.setRefOn(attrs.getValue(i));
        } else if (vAttName.equals("count")){
	    vAns.setCount(Integer.parseInt(attrs.getValue(i)));
        } else if (vAttName.equals("strategy")){
	    vAns.setStrategy(attrs.getValue(i));
	} else {
	    System.out.println("Unkown attribut : Name="+vAttName+" value="+attrs.getValue(i));
        }
      } //end for
    } //end if
    //System.out.println("fin AnalyseArg");
    return vAns;
  } //end proc

    /** Assign the attribute on the current element */
  protected final void Create(TProcess vCurrent, ParserAnswer iPA) {
    //System.out.println("Create " +iPA.getName());
    vCurrent.setName(iPA.getName());
    TProcessComposite vFather = (TProcessComposite) iContext.peek();
    if (vFather!=null) 
	{
	    vFather.addProcess(vCurrent);
	} 
    else 
	{   
	    System.out.println("[ProcessParser] Error , the Father is null");
	}
    
    String[] iOut=iPA.getOut();
    String[] iIn=iPA.getIn();
    int iNbIn = iIn.length;
    int iNbOut = iOut.length;
    vCurrent.setName(iPA.getName());

    //System.out.println("Create " +iPA.getName()+" Nbin= "+iNbIn+" NbOut= "+iNbOut);
    for (int i=0;i<iNbOut; i++) {
      //System.out.println("[create out : " +iPA.getName()+"] "+iOut[i]);
      if ( vCurrent.getOut(i)!=null) {
		  vFather.InsertUnlinkedOutPort(iOut[i],vCurrent.getOut(i));
      	  vCurrent.getOut(i).setCompositeName(iOut[i]);
	  } //for loop
    }

    for (int i=0;i<iNbIn; i++) {
      //System.out.println("[create in : " +iPA.getName()+"] "+iIn[i]);
      vFather.InsertUnlinkedInPort(iIn[i],vCurrent.getIn(i));
      vCurrent.getIn(i).setCompositeName(iIn[i]);
    }

  }

    /** Assign the attribute on the current element */
  protected final void CreateComposite(TProcessComposite vCurrent, ParserAnswer iPA) {
    //System.out.println("Create " +iPA.getName());
    vCurrent.setName(iPA.getName());
    vTemplateList.addProcess(vCurrent);
        
    String[] iOut=iPA.getOut();
    String[] iIn=iPA.getIn();
    int iNbIn = iIn.length;
    int iNbOut = iOut.length;
    vCurrent.setName(iPA.getName());

    //System.out.println("Create " +iPA.getName()+" Nbin= "+iNbIn+" NbOut= "+iNbOut);
    for (int i=0;i<iNbOut; i++) {
      //System.out.println("[ProcessParser:CrComposite][create out : " +iPA.getName()+"] "+iOut[i]);
      vCurrent.InsertCompositeOutPort(iOut[i]);
    }
    
    for (int i=0;i<iNbIn; i++) {
      //System.out.println("[ProcessParser:CrComposite][create in : " +iPA.getName()+"] "+iIn[i]);
      vCurrent.InsertCompositeInPort(iIn[i]);
    }
    
  }
    /** Assign the attribute on the current element */
  protected final void CreateRefComplex(ParserAnswer iPA) {
    //System.out.println("[ProcessParser]Create RefComplexe" +iPA.getName());

    TProcessComposite vFather = (TProcessComposite) iContext.pop();
    ParserAnswer vFatherPA = (ParserAnswer) iContext.pop();
    TProcessComposite vRoot = (TProcessComposite) iContext.peek();
    iContext.push(vFatherPA);
    iContext.push(vFather);
    TProcess vCurrent = null;
   
    vCurrent = vTemplateList.getProcess(iPA.getRefOn());
/*
    if (vCurrent==null) {
      System.out.println("le process pas trouve : "+ iPA.getRefOn());
    } else {
      System.out.println("le process trouve : " +  vTemplateList.getProcess(iPA.getRefOn()).getName());
      
    }
*/
    vCurrent = vCurrent.Copy(vFather);
    vCurrent.setName(iPA.getName()); //+System.currentTimeMillis());

    if (vFather!=null) {
      String[] iOut=iPA.getOut();
      String[] iIn=iPA.getIn();
      int iNbIn = iIn.length;
      int iNbOut = iOut.length;
      
      for (int i=0;i<iNbOut; i++) {
        //System.out.println("[processParser : insert out : " +iPA.getName()+"] "+iOut[i]);
        if (vCurrent.getOut(i)!=null) {
			vFather.removeUnlinkedOutPort(vCurrent.getOut(i).getName());
        	vCurrent.getOut(i).setCompositeName(iOut[i]);
        	vFather.InsertUnlinkedOutPort(iOut[i],vCurrent.getOut(i));
		} //else it's for loop, we don't know out port staticaly
      }
      for (int i=0;i<iNbIn; i++) {
        //System.out.println("[processParser : insert in : " +iPA.getName()+"] "+iIn[i]);
        vFather.removeUnlinkedInPort(vCurrent.getIn(i).getName());
        vCurrent.getIn(i).setCompositeName(iIn[i]);
        vFather.InsertUnlinkedInPort(iIn[i],vCurrent.getIn(i));
      }
    } else {
      System.out.println("[ProcessParser] the Father of the apply-process is null");
    }
  }

}//end class
