/**
 * $Id: RuleSetParser.java,v 1.14 2001/06/13 12:16:53 jerome Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
Author  : Laurent Tardif
Date    : 05 01 2001
Content : Define the rule set parser 
*/
package fr.fluxmedia.transmorpher.Parser ;


// imported SAX Classes 

import org.xml.sax.Attributes;

//Imported JAVA CLasses

import java.util.Vector;


//Imported Transmorpher CLasses 

import fr.fluxmedia.transmorpher.Files.ReadWrite ;
import fr.fluxmedia.transmorpher.Files.Constants ;


public final class RuleSetParser {

/**
  *
  *
  * Construction of stylesheet
  *
  *
  **/

    /** Are we inside a ModTag */
  static boolean insideModTag = false;

  /** Allow to define rule set, this definition build a XSL file */
  protected static final ReadWrite startDefineRuleSet(Attributes attrs) {
    
    //System.out.println("START FILE");
    ReadWrite FileRW = new ReadWrite(attrs.getValue("name")+".xsl",Constants.WRITE); 
    FileRW.Writeln("<?xml version=\"1.0\"?>"); 
    FileRW.Writeln("<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\"");
    FileRW.Writeln(" xmlns:regexp=\"xalan://fr.fluxmedia.transmorpher.regexp.RegularExpression\">");
    FileRW.Writeln("");    
    FileRW.Writeln("<!-- ************************************************************ -->");    
    FileRW.Writeln("<!--        This style sheet was generated by Transmorpher        -->");    
    FileRW.Writeln("<!-- ************************************************************ -->");    
    FileRW.Writeln("");    

    FileRW.Writeln("  <!-- Copying the root and its attributes -->");
    FileRW.Writeln("  <xsl:template match=\"/\">");
    FileRW.Writeln("    <xsl:apply-templates/>");
    FileRW.Writeln("  </xsl:template>");
    FileRW.Writeln("  <xsl:template match=\"@*\">");
    FileRW.Writeln("    <xsl:attribute name=\"{name()}\"><xsl:value-of select=\".\"/>");
    FileRW.Writeln("    </xsl:attribute>");
    FileRW.Writeln("  </xsl:template>");
    FileRW.Writeln("");    
    FileRW.Writeln("  <!-- Copying all elements and attributes -->");
    FileRW.Writeln("  <xsl:template match=\"*\">");
    FileRW.Writeln("    <xsl:copy>");
    FileRW.Writeln("      <xsl:apply-templates select=\"*|@*|text()\"/>");
    FileRW.Writeln("    </xsl:copy>");
    FileRW.Writeln("  </xsl:template>");
    FileRW.Writeln("");    
    FileRW.Writeln("<!-- ************************************************************ -->");    
    FileRW.Writeln("<!--     End of the general section, here begins the stylesheet   -->");    
    FileRW.Writeln("<!-- ************************************************************ -->");    
    FileRW.Writeln("");    

    return FileRW ;
  }

  /** Allow to define a basic transformation : Map an attribute into another one
    <maptag match="<x>" target="<y>" />
 */
  protected static final void startMapTag(ReadWrite iXslFile,Attributes attrs){
    iXslFile.Writeln(2,"<!-- Mapping elements -->");  
    iXslFile.Write(2,"<xsl:template match=\"");
    if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
    iXslFile.Writeln(attrs.getValue("match")+"\">");
    iXslFile.Writeln(4,"<"+attrs.getValue("target")+">");
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"*|@*|text()\"/>");
    iXslFile.Writeln(4,"</"+attrs.getValue("target")+">");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");
  }

  /** Allow to define a basic transformation : remove an element */
  protected static final void startRemTag(ReadWrite iXslFile,Attributes attrs){
    iXslFile.Writeln(2,"<!-- Removing elements "+attrs.getValue("match")+" -->");
    iXslFile.Write(2,"<xsl:template match=\"");
    if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
    iXslFile.Writeln(attrs.getValue("match")+"\"/>");
    iXslFile.Writeln("");  
  }

  /** Allow to define a basic transformation Flatten an element*/
  protected static final void startFlatten(ReadWrite iXslFile,Attributes attrs){
      /**  Flatten specification:
	  (A (x) (A y) (z)) is reduced into (A x y z)
	   This is applied recursivly, i.e. let T being the Flatten transformations it is defiend by:
	   T((A x...y)) -> (A S(x)... S(y))
	   T((B x...y)) -> (B T(x)...T(y)): this is default template
	   S((A x...y)) -> S(x)...S(y)
           S((B x...y)) -> (B T(x)...T(y))
	   T= default mode, S=insideA mode
      */
    iXslFile.Writeln(2,"<!-- Flatten for "+attrs.getValue("match") +" (default mode) -->");  
    iXslFile.Writeln(2,"<xsl:template match=\""+attrs.getValue("match") +"\">");
    iXslFile.Writeln(4,"<xsl:copy>");
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"./*|./text()\" mode=\"insideA\">");
    iXslFile.Writeln(4,"</xsl:copy>");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");    
    iXslFile.Writeln(2,"<xsl:template match=\"*|@*|text()\">");
    iXslFile.Writeln(4,"<xsl:copy>");
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"*|@*|text()\"/>");
    iXslFile.Writeln(4,"</xsl:copy>");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");

    iXslFile.Writeln(2,"<!-- Flatten for "+attrs.getValue("match") +" (inside mode) -->");  
    iXslFile.Writeln(2,"<xsl:template match=\""+attrs.getValue("match") +"\" mode=\"insideA\">");
    iXslFile.Writeln(4,"<xsl:copy>");
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"./*|./text()\" mode=\"insideA\">");
    iXslFile.Writeln(4,"</xsl:copy>");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");    
    iXslFile.Writeln(2,"<xsl:template match=\"*|@*|text()\" mode=\"insideA\">");
    iXslFile.Writeln(4,"<xsl:copy>");
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"*|text()\">");
    iXslFile.Writeln(4,"</xsl:copy>");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");    

  }

  /**  Allow to define a basic transformation : Modify a tag */
  protected static final String startModTag(ReadWrite iXslFile,Attributes attrs) {

    insideModTag = true;
    String iModTagName=attrs.getValue("match");
    iXslFile.Writeln(2,"<!-- Modifying element "+iModTagName+" -->");    
    iXslFile.Write(2,"<xsl:template match=\"");
    if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
    iXslFile.Writeln(iModTagName+"\">");
    iXslFile.Writeln(4,"<xsl:copy>");
    return iModTagName;
  }
  

  /** close the ModTag */
  protected static final void endModTag(Vector iRemAttList,ReadWrite iXslFile) {
    iXslFile.Writeln(6,"<xsl:apply-templates select=\"*|@*|text()\"/>");    
    iXslFile.Writeln(4,"</xsl:copy>");
    iXslFile.Writeln(2,"</xsl:template>");
    iXslFile.Writeln("");
    //Compute all delayed RemAttribute
    RemAllAtt(iRemAttList,iXslFile);
    iXslFile.Writeln("");
    insideModTag = false;
  }

  /** Allow to define a basic transformation change the name of an attribute on an element */
  protected static final void startMapAtt(Vector iRemAttList, String iModTagName,ReadWrite iXslFile,Attributes attrs){
      if ( insideModTag ){
    iXslFile.Writeln(6,"<!-- Mapping attributes "+attrs.getValue("match")+" to "+attrs.getValue("target")+" -->");  
    iXslFile.Writeln(6,"<xsl:attribute name=\""+ attrs.getValue("target")+"\">");
    iXslFile.Write("<xsl:value-of select=\"@"+attrs.getValue("match")+"\"/>"); 
    iXslFile.Writeln(6,"</xsl:attribute>"); 
    iXslFile.Writeln("");
    
    iRemAttList.add(new RemAtt(iModTagName,"@"+attrs.getValue("name") ) );
      } else {
    iXslFile.Writeln(2,"<!-- Mapping attributes "+attrs.getValue("match")+" to "+attrs.getValue("target")+" -->");  
    iXslFile.Write(2,"<xsl:template match=\"");
    if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
    iXslFile.Writeln("@"+ attrs.getValue("match")+"\">");
    iXslFile.Writeln(4,"<xsl:attribute name=\""+ attrs.getValue("target")+"\">");
    iXslFile.Write("<xsl:value-of select=\"@"+attrs.getValue("match")+"\"/>"); 
    iXslFile.Writeln(4,"</xsl:attribute>"); 
    iXslFile.Writeln(2,"</xsl:template>"); 
    iXslFile.Writeln("");
      }
  }



  /** Allow to define a basic transformation remove a tag on element */
  protected static final void startRemAtt(Vector iRemAttList, String iModTagName,ReadWrite iXslFile,Attributes attrs){
      if ( insideModTag ){
    iRemAttList.add(new RemAtt(iModTagName,"@"+attrs.getValue("match") ) );
      } else {
    iXslFile.Writeln(2,"<!-- Removing attributes "+attrs.getValue("match")+" -->");  
    iXslFile.Write(2,"<xsl:template match=\"");
    if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
    iXslFile.Writeln("@"+attrs.getValue("match")+"\"/>");
    iXslFile.Writeln("");
      }
  }


  /** Apply the transformation for all RemAtt, which aredelayed */
  protected static final void RemAllAtt(Vector iRemAttList,ReadWrite iXslFile){
    int Size = iRemAttList.size() ;
    for (int i=0; i< Size; i++) {
      ComputeRemAtt(iXslFile,(RemAtt)iRemAttList.remove(0));  
    }
  }
  
  /** Apply the transformation for One RemAtt, which aredelayed */
  protected static final void ComputeRemAtt(ReadWrite iXslFile,RemAtt pRemAtt){
    iXslFile.Writeln(2,"<!-- Removing attributes "+ pRemAtt.getAttributeName() 
                     +" on elements "+pRemAtt.getElementName()+" -->");
    iXslFile.Writeln(4,"  <xsl:template match=\""+pRemAtt.getElementName()+"/"+pRemAtt.getAttributeName()+"\"/>");
    iXslFile.Writeln(2,"");
  }

  /** Allow to define a basic transformation : add a tag on element */
  protected static final void startAddAtt(ReadWrite iXslFile,Attributes attrs){
    iXslFile.Writeln(6,"<!-- Adding attributes "+ attrs.getValue("name")+" -->");      
    iXslFile.Write(6,"<xsl:attribute name=\""+ attrs.getValue("name")+"\">");
    iXslFile.Write(attrs.getValue("value"));
    iXslFile.Writeln(6,"</xsl:attribute>"); 
    iXslFile.Writeln("");
  }

 protected static final void extend(ReadWrite iXslFile,Attributes attrs){
     
     iXslFile.Writeln(2,"<!-- Importing another ruleset -->");      
     iXslFile.Writeln(2,"<xsl:import href=\""+attrs.getValue("ref")+".xsl\"/>");		 
     iXslFile.Writeln("");
 }
    
    protected static final void endExtend(ReadWrite iXslFile){
    }

    protected static final void startResubst(ReadWrite iXslFile,Attributes attrs){
	
	String source = attrs.getValue("source");
	String target = attrs.getValue("target");
	int arob = attrs.getValue("match").indexOf('@');
	
	// later they could have been some changes in this function
	String apply = "all";

//	if(attrs.getLength()==4)
	apply = attrs.getValue("apply");

        iXslFile.Writeln(2,"<!-- Substituting "+apply+" "+source+" by "+target+" in "+ attrs.getValue("match")+" -->");      
	iXslFile.Write(2,"<xsl:template match =\"");
        if( attrs.getValue("context") != null ) iXslFile.Write(attrs.getValue("context")+"/");
	iXslFile.Writeln(attrs.getValue("match")+"\">");
	iXslFile.Writeln(4,"<xsl:if test=\"function-available('regexp:substitute') and function-available('regexp:substituteAll') \">");
        
        if ( arob > 0 ) iXslFile.Writeln(6,"<xsl:attribute name=\""+(attrs.getValue("match")).substring(arob+1)+"\">");
	if (apply.equals("once"))
	iXslFile.Writeln(8,"<xsl:value-of select=\"regexp:substitute(.,'"+source+"','"+target+"')\"/>");
	else
        iXslFile.Writeln(8,"<xsl:value-of select=\"regexp:substituteAll(.,'"+source+"','"+target+"')\"/>");
        if ( arob > 0 ) iXslFile.Writeln(6,"</xsl:attribute>"); 
    
        iXslFile.Writeln(4,"</xsl:if>");
	
    }

    protected static final void endResubst(ReadWrite iXslFile){
    	iXslFile.Writeln(2,"</xsl:template>");
    	iXslFile.Writeln("");
    }

  /** close the iXlFile */
  protected static final void endDefineRuleSet(ReadWrite iXslFile) {
    iXslFile.Writeln("</xsl:stylesheet>") ;
    iXslFile.Close();    
  }

}//end class
