/**
 * $Id: TLoop.java,v 1.3 2001/06/12 15:50:30 gchomat Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  * This class provide a basic process which is
  * able to read an XML file
  *
  * @author Laurent.Tardif@inrialpes.fr
  * @since jdk 1.3 && SAX 2.0
  */

package fr.fluxmedia.transmorpher.TProcessFactory ;


// Imported SAX classes

import org.xml.sax.SAXException;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.XMLReader;

// Imported TraX classes

import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;


// Imported java.io classes

import java.io.IOException;

public final class TLoop
  extends TProcessComposite {

  int iNbLoop = 0;
  TProcess vCurrent = null;
  TProcess vSource = null;
  String[] iIn = null ;
  String[] iOut = null;
  int iCount=-1;
  String iRefOn = null;
  TProcessComposite iTemplateList = null ;
  static int ID =  0;
    
  /** The constructor */
  public TLoop (String[] pIn, String[] pOut, String pRefOn, int pCount,TProcessComposite pTemplateList)
  throws TransformerException,
         TransformerConfigurationException,
         SAXException,
         IOException {
    super(pIn,pOut);
    iIn=pIn;
    iOut=pOut;
    iRefOn=pRefOn;
    iTemplateList = pTemplateList ;
    iCount = pCount;
    vSource = pTemplateList.getProcess(pRefOn);
      
    if (pCount ==1) {
      buildAlone();
      createLoopLink();
    } else if (pCount < TProcessFactory.INFINITY) {
      buildFirstOne();
      buildOtherOne();
      buildLastOne();
      createLoopLink();
      //System.out.println("fin rneomage liens du composite ");
    } else {
      // infinity loop until .....
      System.out.println("finite loop ? i hope :)");
      buildConditionnalLoop();
    } //end if

  } //end constructor

  private final void buildAlone() {
      // --------------------------------------------------------------------
      //Copy the first One
      // --------------------------------------------------------------------
        System.out.println("Copy the first one "+vCurrent);
        vCurrent = vSource.Copy(this);
        vCurrent.setName("InsideLoop"+System.currentTimeMillis());
        for (int j=0;j<vCurrent.getOut().length; j++) {
          this.removeUnlinkedOutPort(vCurrent.getOut(j).getName());
          vCurrent.getOut(j).setCompositeName(iOut[j]);
          vCurrent.getOut(j).setName(iOut[j]);
          this.InsertUnlinkedOutPort(iOut[j],vCurrent.getOut(j));
          this.InsertCompositeOutPort(iOut[j]);
          //System.out.println("[processParser : jnsert out : " +vCurrent.getOut(j).getName()+"] ");
        } //end for
      
        for (int j=0;j<vCurrent.getIn().length; j++) {
          this.removeUnlinkedInPort(vCurrent.getIn(j).getName());
          vCurrent.getIn(j).setCompositeName(iIn[j]);
          vCurrent.getIn(j).setName(iIn[j]);
          this.InsertUnlinkedInPort(iIn[j],vCurrent.getIn(j));
          this.InsertCompositeInPort(iIn[j]);
          //System.out.println("[processParser : jnsert in : " +vCurrent.getIn(j).getName()+"] ");
        } //end for
        //this.addProcess(vCurrent);

    } //end build firstOne



  private final void buildFirstOne() {
      // --------------------------------------------------------------------
      //Copy the first One
      // --------------------------------------------------------------------
        System.out.println("Copy the first one "+vCurrent);
        vCurrent = vSource.Copy(this);
        vCurrent.setName("InsideLoop_"+ID++);
        for (int j=0;j<vCurrent.getOut().length; j++) {
          this.removeUnlinkedOutPort(vCurrent.getOut(j).getName());
          vCurrent.getOut(j).setCompositeName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop)+"_"+j);
          vCurrent.getOut(j).setName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop++)+"_"+j);
          this.InsertUnlinkedOutPort(vCurrent.getOut(j).getCompositeName(),vCurrent.getOut(j));
          //System.out.println("[processParser : jnsert out : " +vCurrent.getOut(j).getName()+"] ");
        } //end for
      
        for (int j=0;j<vCurrent.getIn().length; j++) {
          this.removeUnlinkedInPort(vCurrent.getIn(j).getName());
          vCurrent.getIn(j).setCompositeName(iIn[j]);
          vCurrent.getIn(j).setName(iIn[j]);
          this.InsertUnlinkedInPort(iIn[j],vCurrent.getIn(j));
          this.InsertCompositeInPort(iIn[j]);
          //System.out.println("[processParser : jnsert in : " +vCurrent.getIn(j).getName()+"] ");
        } //end for
        //this.addProcess(vCurrent);

    } //end build firstOne
  
  private final void buildOtherOne() {
      // --------------------------------------------------------------------
      //Copy the other
      // --------------------------------------------------------------------
      System.out.println("Copy the others ");
      for (int i=1;i<iCount-1; i++) {
        vCurrent = vSource.Copy(this);
        vCurrent.setName("Loop"+System.currentTimeMillis());
        
        for (int j=0;j<vCurrent.getIn().length; j++) {
          //System.out.println("[processParser : jnsert in ");
          this.removeUnlinkedInPort(vCurrent.getIn(j).getName());
          vCurrent.getIn(j).setCompositeName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop-1)+"_"+j);
          vCurrent.getIn(j).setName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop-1)+"_"+j);
          this.InsertUnlinkedInPort(vCurrent.getIn(j).getCompositeName(),vCurrent.getIn(j));
          //System.out.println("[processParser : jnsert in : " +vCurrent.getIn(j).getName()+"] ");
        } //end for
        
        for (int j=0;j<vCurrent.getOut().length; j++) {
          this.removeUnlinkedOutPort(vCurrent.getOut(j).getName());
          vCurrent.getOut(j).setCompositeName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop)+"_"+j);
          vCurrent.getOut(j).setName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop++)+"_"+j);
          this.InsertUnlinkedOutPort(vCurrent.getOut(j).getCompositeName(),vCurrent.getOut(j));
          //System.out.println("[processParser : jnsert out : " +vCurrent.getOut(j).getName()+"] ");
        } //end for
      } //end for
      
  } //end buildOtherOne
  

  private final void buildLastOne() {
  
        // --------------------------------------------------------------------
      //Copy the last one
      // --------------------------------------------------------------------
      System.out.println("Copy the last one ");
      vCurrent = vSource.Copy(this);
      vCurrent.setName("Loop_"+ID++);
        for (int j=0;j<vCurrent.getIn().length; j++) {
          this.removeUnlinkedInPort(vCurrent.getIn(j).getName());
          vCurrent.getIn(j).setCompositeName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop-1)+"_"+j);
          vCurrent.getIn(j).setName("Loop_"+vCurrent.getOut(j).getName()+"_"+(iNbLoop-1)+"_"+j);
          this.InsertUnlinkedInPort(vCurrent.getIn(j).getCompositeName(),vCurrent.getIn(j));
          //System.out.println("[processParser : jnsert in : " +vCurrent.getIn(j).getName()+"] ");
        }//end for

        for (int j=0;j<vCurrent.getOut().length; j++) {
          this.removeUnlinkedOutPort(vCurrent.getOut(j).getName());
          vCurrent.getOut(j).setCompositeName(iOut[j]);
          vCurrent.getOut(j).setName(iOut[j]);
          this.InsertUnlinkedOutPort(iOut[j],vCurrent.getOut(j));
          this.InsertCompositeOutPort(iOut[j]);
          //System.out.println("[processParser : jnsert out : " +vCurrent.getOut(j).getName()+"] ");
        } //end for
  
  }
  
  
  public final void createLoopLink()
  throws SAXException {
      //System.out.println("On va faire les liens ");
      this.CreateLink();
      //System.out.println("fin liens ");
       
      //System.out.println("On va faire les liens du composite ");
      this.LinkCompositePort();
      //System.out.println("fin liens du composite ");

      //set the composite name for all the composite port
      for (int i=0;i<iOut.length; i++) {
        //System.out.println("[ Porcess COmposite][rename [ "+getName()+"] out : " +iListOut[i].getName()+"] "+iListOut[i]);
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] out : " +iListOut[i].getCompositeName()+"] "+iListOut[i]);
        this.getOut(i).setCompositeName(iOut[i]);
        this.getOut(i).setName(iOut[i]);
      } //end for
      for (int i=0;i<iIn.length; i++) {
        //System.out.println("[ Porcess COmposite][rename dans [ "+getName()+"] in : " +iListIn[i].getCompositeName()+"] "+iListIn[i]);
        this.getIn(i).setCompositeName(iIn[i]);
        this.getIn(i).setName(iIn[i]);
      } //end for

  
  } //end createLoopLink
  

  public final void buildConditionnalLoop()
  throws TransformerException,
         TransformerConfigurationException,
         SAXException,
         IOException {
          
    String[] PortNameIn = new String [iIn.length];
    String[] PortNameOut = new String [iOut.length];
    int MyID = ID ++ ;
    
    int LoopId = MyID ;
    
    for (int i=0;i<iIn.length; i++) {
      PortNameIn[i]  =  iIn[i];
      PortNameOut[i] = iOut[i];
    }
    TLoopUtility myLoopUtility = new TLoopUtility(vSource,this, PortNameIn, PortNameOut);
/*
    for (int i=0;i<iIn.length; i++) {
      this.InsertUnlinkedInPort(iIn[i],myLoopUtility.getIn(i));
      this.InsertCompositeInPort(iIn[i]);
    }*/
          
    //createLoopLink();
  }
  
  
  public final TProcess internalCopy() {
    //System.out.println("INTERNAL COPY " + getName());
    TProcessComposite vAns = null;
    String name ;
    TProcess vDummy = null;
    String[] vPortName = new String [getOut().length];

    for (int i =0; i<vPortName.length; i++) {
      //vPortName[i]=out[i].getCompositeName();
		if(iListOut[i]!=null)vPortName[i]=iListOut[i].getName();
    }


    String[] vPortNameIn = new String [getIn().length];

    for (int i =0; i<vPortNameIn.length; i++) {
      //vPortNameIn[i]=in[i].getCompositeName();
      vPortName[i]=iListIn[i].getName();
    }

    try {
      vAns = new TLoop(iIn, iOut, iRefOn, iCount,iTemplateList);
      vAns.setName(getName());
      //TODO : il faut creer qqc
    } catch (Exception e) {
      System.out.println("Erreur lors de la creation d'un TProcessComposite: " +e);
    } //end catch Exception e
    return vAns ;
  } //end internal copy

} //end class
