/**
 * $Id: TProcessComposite.java,v 1.5 2001/06/12 15:50:30 gchomat Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  * This class allow to build a composite process
  *
  * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
  * @see fr.fluxmedia.transmorpher.TProcessFactory.XML_Port
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;

// Imported TraX classes

import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;


// Imported SAX classes

import org.xml.sax.SAXException;


// Imported JAVA API for XML Parsing 1.0 classes

import java.io.IOException;

import java.util.Hashtable ;
import java.util.Enumeration ;
import java.util.Vector ;

public class TProcessComposite
   extends TProcess_Impl {

  /** The max number of process in a composite */
  public final static int NB_PROCESS_RISE = 200;

  /** The list of unlinked in XML_Port */
  Hashtable iListOutNotAffected = new Hashtable(NB_PROCESS_RISE*5);

  /** The list of unlinked in XML_Port */
  Vector iListCompositeOut = new Vector();

  /** The list of unlinked out XML_Port */
  Hashtable iListInNotAffected = new Hashtable(NB_PROCESS_RISE*5);

  /** The list of child process */
  Vector iListCompositeIn = new Vector();

  /** The lsit of child process */
  Vector iProcessList = new Vector(NB_PROCESS_RISE,NB_PROCESS_RISE/5);
  

  /** The constructor : build a Composite process with pNbIn in Port, and pNbOut Out Port  */
  public TProcessComposite(String[] pIn, String[] pOut)
    throws TransformerException, TransformerConfigurationException,
           SAXException, IOException	 {

     super(pIn,pOut);
    //System.out.println("{ TProcess] creation composite");

  } //end constructor


  /** Insert a new unlinked IN port, with its name */
  public final void InsertUnlinkedInPort(String pName, XML_Port pPort) {
    //System.out.println("[ProcessComposite ]"+getName()+". InsertUnlinkedInPort : name="+pName+" Port= "+pPort );
    iListInNotAffected.put(pName,pPort);
  }

  /** Insert a new unlinked Out port, with its name */
  public final void InsertUnlinkedOutPort(String pName, XML_Port pPort) {
    //System.out.println("[ProcessComposite ]"+getName()+". InsertUnlinkedOutPort: name="+pName+" Port= "+pPort );
    iListOutNotAffected.put(pName,pPort);
  }


 /** remove an unlinked IN port, with its name */
  public final void removeUnlinkedInPort(String pName) {
    //System.out.println("[ProcessComposite ]"+getName()+". removeUnlinkedInPort : name="+pName);
    iListInNotAffected.remove(pName);
  }

  /** remove an unlinked Out port, with its name */
  public final void removeUnlinkedOutPort(String pName) {
    //System.out.println("[ProcessComposite ]"+getName()+". removeUnlinkedOutPort: name="+pName );
    iListOutNotAffected.remove(pName);
  }


  public final void InsertCompositeOutPort(String pName) {
    //System.out.println("[ProcessComposite ] "+getName()+".InsertUnlinkedOutCompositePort: name="+pName);
    iListCompositeOut.add(pName);
  }
  public final void InsertCompositeInPort(String pName) {
    //System.out.println("[ProcessComposite ] "+getName()+".InsertUnlinkedInCompositePort: name="+pName);
    iListCompositeIn.add(pName);
  }

  /** Add a process to the child process list */
  public final void addProcess(TProcess pProcess) {
      //System.out.println("[CompositeProcess] Ajout un process : " +pProcess.getName());
    iProcessList.add(pProcess);
  }


  /** Get the process with the given name */

    public final TProcess getProcess(String vName) {
	for (Enumeration e = iProcessList.elements(); e.hasMoreElements();) {
	    TProcess vProcess = (TProcess)e.nextElement();
	    if(vProcess.getName().equals(vName))
		return vProcess;
	}
	return null;
    }

  /** Call it at the end of the addProcess call, to link all the children */
  public final void CreateLink() throws  SAXException   {
    //System.out.println("[ Porcess COmposite]Create Link");
    XML_Port p1 = null;
    XML_Port p2 = null;
    for (Enumeration e = iListInNotAffected.keys() ; e.hasMoreElements() ;) {
      String v = (String)e.nextElement() ;
      p1 = (XML_Port) iListOutNotAffected.get(v);
      p2 = (XML_Port) iListInNotAffected.get(v);
      //System.out.println("**************");
      //System.out.println("[ Porcess COmposite]"+getName()+".In  ["+ v + "]  = " +p1);
      //System.out.println("[ Porcess COmposite]"+getName()+".Out ["+ v + "]  = " +p2);
      //System.out.println("--------------");
      if ((p1!=null) && (p2!=null)) {
        XML_Port.link(p1,p2); //else it must be a compiste port
        iListInNotAffected.remove(v);
        iListOutNotAffected.remove(v);
      }
      
    } //fin du for
  } //fin CreateLink

  public final void LinkCompositePort() {
    //System.out.println("[ TProcessComposite]"+getName()+".Create Link composite Port");
    String name = null;
    for (int i=0; i<iListCompositeIn.size();i++) {
      name = (String) iListCompositeIn.elementAt(i);
      //System.out.println("[ Porcess COmposite]"+getName()+"[LinkPort]In  ["+ name + "]");
      iListIn[i] = (XML_Port) iListInNotAffected.get(name);
      if (iListIn[i]!=null)iListIn[i].setCompositeName(name); //for loop
      //System.out.println("[ Porcess COmposite]"+getName()+"[LinkPort]In  ["+ name + "]port="+iListIn[i]);
    }

    for (int i=0; i<iListCompositeOut.size();i++) {
      name = (String) iListCompositeOut.elementAt(i);
      //System.out.println("[ Porcess COmposite]"+getName()+"[LinkPort]Out  ["+ name + "]");
      iListOut[i] = (XML_Port) iListOutNotAffected.get(name);
      if (iListOut[i]!=null) {
		  iListOut[i].setCompositeName(name);
	  } //else must be for loop
		
      //System.out.println("[ TProcessComposite]"+getName()+"[LinkPort]Out  ["+ name + "] port="+iListOut[i]);
    }

    //System.out.println("[ TProcessComposite]Fin Create Link composite Port");
  }


  /** copy of the composite. This is the complex method of the TProcess classes.
      Instead we must copy all the child and linked them correctly. This method
      is call by the copy method of the XMLProcess_impl classes
  */

  public TProcess internalCopy() {
    //System.out.println("INTERNAL COPY " + getName());
    TProcessComposite vAns = null;
    String name ;
    TProcess vDummy = null;
    String[] vPortName = new String [getOut().length];

    for (int i =0; i<vPortName.length; i++) {
      //vPortName[i]=out[i].getCompositeName();
      if (iListOut[i]!=null) vPortName[i]=iListOut[i].getName();
		//for loop
    }


    String[] vPortNameIn = new String [getIn().length];

    for (int i =0; i<vPortNameIn.length; i++) {
      //vPortNameIn[i]=in[i].getCompositeName();
      vPortName[i]=iListIn[i].getName();
    }

    try {
      vAns = new TProcessComposite(vPortNameIn,vPortName);
      vAns.setName(getName());
      //Copy all children and insert them in the list of process
 
	for (Enumeration e = iProcessList.elements(); e.hasMoreElements();) {
	    TProcess vProcess = (TProcess)e.nextElement();
	    if (vProcess!=null) 
		vDummy=vProcess.Copy(vAns);
	}

      //insert all the port in the composite

      for (int i=0;i<iNbOut; i++) {
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] out : " +iListOut[i].getName()+"] "+iListOut[i]);
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] out : " +iListOut[i].getCompositeName()+"] "+iListOut[i]);
		  if(iListOut[i]!=null) {
			vAns.InsertCompositeOutPort(iListOut[i].getCompositeName());
		  } //else it's for loop, the out is not know staticaly
      }
      for (int i=0;i<iNbIn; i++) {
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] in : " +iListIn[i].getName()+"] "+iListIn[i]);
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] in : " +iListIn[i].getCompositeName()+"] "+iListIn[i]);
        vAns.InsertCompositeInPort(iListIn[i].getCompositeName());
      }
      
      //link all the port
      
      vAns.CreateLink();
      vAns.LinkCompositePort();

      //set the composite name for all the composite port

      for (int i=0;i<iNbOut; i++) {
		    //System.out.println("[ Porcess COmposite]");
        //System.out.println("[ Porcess COmposite][rename [ "+getName()+"] out : " +iListOut[i].getName()+"] "+iListOut[i]);
        //System.out.println("[ Porcess COmposite][Icopy insert dans [ "+getName()+"] out : " +iListOut[i].getCompositeName()+"] "+iListOut[i]);
        if (vAns.getOut(i)!=null) {
		  vAns.getOut(i).setCompositeName(iListOut[i].getCompositeName());
		} //else We are in loop case, the out port are not staticaly know
      }
      for (int i=0;i<iNbIn; i++) {
    		//System.out.println("[ Porcess COmposite]");
        //System.out.println("[ Porcess COmposite][rename dans [ "+getName()+"] in : " +iListIn[i].getCompositeName()+"] "+iListIn[i]);
        vAns.getIn(i).setCompositeName(iListIn[i].getCompositeName());
      
      }

      //TODO : il faut creer qqc
    } catch (Exception e) {
      System.out.println("Erreur lors de la creation d'un TProcessComposite: " +e);
    }
    return vAns ;
  }


} //end class
