/**
 * $Id: TProcessFactory.java,v 1.6 2001/06/12 15:50:30 gchomat Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  * 
  * This class allow to build the process description 
  * And allow the user to start the resolution  
  * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;


// Imported TraX classes

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXResult;

// Imported TraX classes

import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

// Imported SAX classes
import org.xml.sax.SAXException;

// Imported Java Classes 
import java.io.IOException;
import java.util.Vector ;
import java.util.Enumeration ; 

public final class TProcessFactory {

  static final int NB_READER_RISE = 100;
  static final int INFINITY = 100000 ;
 
  Vector iGeneratorList = new Vector(NB_READER_RISE,NB_READER_RISE/5);

   /** The Transformer factory */  
  static TransformerFactory tFactory =null;
  
  /** the SAXTransformerFactory */
  static SAXTransformerFactory saxTFactory = null ;

  /** Class constructor */
  static {
   tFactory = TransformerFactory.newInstance();
   if (tFactory.getFeature(SAXSource.FEATURE) && tFactory.getFeature(SAXResult.FEATURE)) { 
     // Cast the TransformerFactory to SAXTransformerFactory.
     saxTFactory = ((SAXTransformerFactory) tFactory);
    }
    
  }


  /** The constructor */
  public TProcessFactory ()    
   throws TransformerException, TransformerConfigurationException, 
          SAXException, IOException	{
    //Do nothing :)
    //Very usefull :)))
  }
  
  /** Return the sax factory */
  static final SAXTransformerFactory getSAXTFactory () {
    return saxTFactory;
  }
  
    
  
  /**
    Create a new generator. Currently, this generator is a TReader.
    In the futur, it will be suitable to create different generator according to the type.
    ps : may be, the paramer must be a ParserAnswer instead of String and String[]
    
  */
    public final TProcess newGenerator(String pType,
				       String[] pOut,
				       String pFilename) 
	throws SAXException,
	IOException{
       
	TReader vTReader = new TReader(pOut,pFilename);
	iGeneratorList.add(vTReader);
	return vTReader;
    }
  
   
  /**
    Create a new generator. Currently, this generator is a TReader.
    In the futur, it will be suitable to create different generator according to the type.
    ps : may be, the paramer must be a ParserAnswer instead of String and String[]
  
  */
  public final TProcess newLoop(String[] pIn,
                                String[] pOut,
                                String pRefOn,
                                int pCount,
                                TProcessComposite pTemplateList) 
  throws TransformerException,
         TransformerConfigurationException,
         SAXException,
         IOException{
     return new TLoop(pIn,pOut,pRefOn,pCount,pTemplateList);
  }
  
  
  /** 
    Create a new composite process 
  */  
  public static final TProcess newComposite(String[] pIn,
                                               String[] pOut) 
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException {
    return new TProcessComposite(pIn,pOut);
  }
  
  /**
    Build a new dispatcher. currently only build a basic one. In the next version more complex
    dispatcher can be developed. (With filter, ...)
  */
  public static final TProcess newDispatcher(String pType,
                                                String[] pIn,
                                                String[] pOut) {
    return new TDispatcher(pIn,pOut);
  }


  /**
    Build a new Transformer according th the given type. Currently only
    build  transformer with a XSL file in parameter.  
  */
  public static final TProcess newTransformer(String pType,
                                                 String[] pIn,
                                                 String[] pOut,
                                                 String pFilename)
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException  {
    return new TTransformer(pIn,pOut,pFilename);
  }

 /**
    Build a new query. currently only build a internal one. In the next version more complex
    query can be developp. 
  */
  public static final TProcess newQuery(String pType,
                                           String[] pIn,
                                           String[] pOut,
                                           String pFilename)
    throws TransformerException,
           SAXException,
           IOException  {
    return new TQuery(pType,pIn,pOut,pFilename);
  }

  /** Build an new ruleset process */
  public static final TProcess newRuleSet(String[] pIn,
                                             String[] pOut,
                                             String pRefOn)
    throws TransformerException,
           TransformerConfigurationException,
           SAXException,
           IOException  {
    return new RuleSet(pIn,pOut,pRefOn);
  }
  
  
    /**
    Build a new Connector. currently only build a basic one. In the next version more complex
    Connector can be developed. (With filter, ...)
  */
  public static final TProcess newConnector(String pType,
                                                String[] pIn,
                                                String[] pOut) {
    return new TConnector(pIn,pOut);
  }

  /**
    Build a new Connector. currently only build a basic one. In the next version more complex
    Connector can be developp. (With filter, ...)
  */

  public static final TProcess newSerializer(String pType,
                                                String[] pIn,
                                                String pFilename)
    throws SAXException,
           IOException {
    return new TSerializer(pIn,pFilename);
  }  
 
  public final void execProcess()
  throws SAXException,
         IOException{
    //Call the exec process on all generator   
     for (Enumeration e = iGeneratorList.elements() ; e.hasMoreElements() ;) {
	 ((TReader)e.nextElement()).execProcess();
        
    }
  }
}

