/**
 * $Id: XML_Port.java,v 1.3 2001/06/12 15:50:30 gchomat Exp $
 *
 * Transmorpher
 * 
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  *
  *
  * @author laurent.Tardif@inrialpes.fr
  * @since SAX 2.0 && jdk 1.3
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;

import org.xml.sax.ContentHandler;

// Imported TraX classes
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.sax.SAXResult;

// Imported SAX classes 
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;


// Imported Serializer classes
import org.apache.xalan.serialize.Serializer;
import org.apache.xalan.transformer.TransformerImpl;

public final class XML_Port {

  /** The name of the port */
  String iName = null;  

  /** The name of the port in his father, available only for composite element */
  String iCompositeName = null;
  
  /** constant for IN XML_PORT */
  public static final int IN  = 0 ;

  /** constant for OUT XML_PORT */
  public static final int OUT = 1 ;
  
  /** constant for print IN && OUT XML_PORT */
  public static final String [] InOut = {"In" , "Out" } ;
  
  
  /** Constant for XML READER */
  public static final int XML_READER              =  0 ;

  /** Constant for NOT AFFECTED PORT */
  public static final int EMPTY                   =  1 ;
  
  /** Constant for XML SERIALIZER */
  public static final int SERIALIZER              =  2 ;  

  /** Constant for  TRANSFORMER_HANDLER_IN */
  public static final int TRANSFORMER_HANDLER_IN  =  3 ;

  /** Constant for TRANSFORMER_HANDLER_OUT  */
  public static final int TRANSFORMER_HANDLER_OUT =  4 ;

  /** Constant for CONTENT_DISPATCHER_IN  */
  public static final int CONTENT_DISPATCHER_IN   =  5 ;    

  /** Constant for CONTENT_DISPATCHER_OUT  */
  public static final int CONTENT_DISPATCHER_OUT  =  6 ;

  /** Constant for CONTENT_HANDLER_IN  */
  public static final int CONTENT_HANDLER_IN      =  7 ;

  /** Constant for CONTENT_HANDLER_OUT */
  public static final int CONTENT_HANDLER_OUT     =  8 ;

  /** Constant for CONNECTOR_IN  */
  public static final int CONNECTOR_IN            =  9 ;

  /** Constant for CONNECTOR_OUT  */
  public static final int CONNECTOR_OUT           = 10 ;

  /** Constant for printing PortType  */
  public static final String [] PortType = {"XML_READER","EMPTY","SERIALIZER",
                                            "TRANSFORMER_HANDLER_IN","TRANSFORMER_HANDLER_OUT",
                                            "CONTENT_DISPATCHER_IN", "CONTENT_DISPATCHER_OUT",
                                            "CONTENT_HANDLER_IN","CONTENT_HANDLER_OUT",
                                            "CONNECTOR_IN","CONNECTOR_OUT"};
  
  /** The XML TProcess Owner */  
  TProcess iProcessOwner = null;
  
  /** The port number */
  int iNumPort =-1 ; 
  
  /** the Type of the port : IN or OUT */
  int iInOut = -1 ;
  
  /** The Type of the owner */
  int iTypePort = -1 ;


  /** The encoding type of the flow */  
  String iEncodingType = "ISO-8859-1";
  
  /** the dtd of the port */
  String iDTDName ="Default_dtd" ;
  

  /** The content flow of the port */
  Object iContent = null; 

  
  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   int pNumPort, int pInOut) {
    iName=pName;
    iCompositeName = pName ;  //by default name = composite name, 
                              //for compostie you must call SET COMPOSITE NAME later :)
    iProcessOwner = pProcessOwner ;
    iNumPort  = pNumPort ;
    iInOut    = pInOut ;
    iTypePort = EMPTY ;
  }  
  
  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   XMLReader pXMLReader,
                   int pNumPort, int pInOut) {
    this(pName,pProcessOwner,pNumPort,pInOut);
    iTypePort =  XML_READER; 
    iContent = pXMLReader;
    //System.out.println("Creation d un port de type XML_READER");
  }

  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   TDispatcher pTDispatcher,
                   int pNumPort, int pInOut) {
                    
    this(pName,pProcessOwner,pNumPort,pInOut);
    if (pInOut==IN) iTypePort =  CONTENT_DISPATCHER_IN;
       else  iTypePort =  CONTENT_DISPATCHER_OUT;
    iContent = pTDispatcher;
    //System.out.println("Creation d un port de type CONTENT_DISPATCHER");
  }    
  
  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   ContentHandler pContentHandler,
                   int pNumPort, int pInOut) {
                    
    this(pName,pProcessOwner,pNumPort,pInOut);
    iContent = pContentHandler;
    iTypePort = CONTENT_HANDLER_IN;
    //System.out.println("Creation d un port de type ContentHandler");  
  }  


  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   TConnector pContentHandler,
                   int pNumPort, int pInOut) {
                    
    this(pName,pProcessOwner,pNumPort,pInOut);
    iContent = pContentHandler;
    iTypePort = CONTENT_HANDLER_OUT;
    //System.out.println("Creation d un port de type ContentHandler");  
  }   
 
  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   TLoopStack1 pContentHandler,
                   int pNumPort, int pInOut) {
                    
    this(pName,pProcessOwner,pNumPort,pInOut);
    iContent = pContentHandler;
    iTypePort = CONTENT_HANDLER_OUT;
    //System.out.println("Creation d un port de type ContentHandler");  
  }   

  
  /** One constructor */
  public XML_Port (String pName,
                   TProcess pProcessOwner,
                   TransformerHandler pTransformerHandler,
                   int pNumPort, int pInOut) {
                    
    this(pName,pProcessOwner,pNumPort,pInOut);
    if (pInOut==IN) iTypePort =  TRANSFORMER_HANDLER_IN;
      else  iTypePort =  TRANSFORMER_HANDLER_OUT;
    iContent = pTransformerHandler;
    //System.out.println("Creation d un port de type TRANSFORMER_HANDLER");  
  }   
  
  /** Get the name of the port */
  public final String getName() {
    return iName;
  }
  
  /** Get the composite name of the port */
  public final String getCompositeName() {
    return iCompositeName;
  }

  /** Set the composite name of the port */    
  public final void setCompositeName(String pCompositeName) {
    iCompositeName = pCompositeName;
  }    
   /** Set the composite name of the port */    
  public final void setName(String pName) {
    iName = pName;
  }  
     
  /** Get the owner of the port */
  public final TProcess getOwner() {
    return iProcessOwner;
  }
  
  /** Get the IN/ OUT type of the port */
  public final int getInOut() {
    return iInOut;
  }

  /** Get the owner type */
  public final int getType() {
    return iTypePort ;
  }
  
  /** Get the port number */
  public final int getNumPort() {
    return iNumPort;
  }
  
  /** get the content */
  public final Object getContent() {
    return iContent;
  }

  /** Set the encoding type of the flow */  
  public final void setEncoding(String pEncoding ) {   
    iEncodingType = pEncoding;
  }

  /** get the encoding type of the flow */  
  public final String getEncoding() {   
    return iEncodingType ;
  }  
   
  
  /** set the dtd of the port */
  public final void setDTD(String pDTDName ) {   
    iDTDName = pDTDName;
  }

  /** get the dtd of the port */
  public final String getDTD() {   
    return iDTDName ;
  }  

  /** Link to port */
  public final static void link(XML_Port p1, XML_Port p2) throws SAXException {
    if (p2.getType()==EMPTY) {
      //System.out.println("Link : p2 is EMPTY reverse the call order");
      link(p2,p1);
    }
    
    switch (p1.getType()) {
      case XML_READER :
        //System.out.println("Link un XML_READER");
        ((XMLReader)p1.getContent()).setContentHandler((ContentHandler)p2.getContent());
        break ;
      case EMPTY :
        //System.out.println("Link un EMPTY");    
        TProcess p1_process = p1.getOwner() ;
        if (p1.getInOut() == IN ) {
          p1_process.setIn(p1.getNumPort() , p2);
        } else {
          p1_process.setOut(p1.getNumPort() , p2);
        }
        break ;
      case SERIALIZER :
        System.out.println("Link un SERIALIZER not yet implemented !");
        break;
      case TRANSFORMER_HANDLER_IN :
        System.out.println("Link un TRANSFORMER_HANDLER_IN not yet implemented !");
        break;
      case TRANSFORMER_HANDLER_OUT :
        //System.out.println("Link un TRANSFORMER_HANDLER_OUT");
        TransformerHandler TH=  ((TransformerHandler)p1.getContent());
        TH.setResult(new SAXResult((ContentHandler)p2.getContent()));
        break;
      case CONTENT_DISPATCHER_IN :
        System.out.println("Link un CONTENT_DISPATCHER_IN not yet implemented !");
        break;
      case CONTENT_DISPATCHER_OUT :
        //System.out.println("Link un CONTENT_DISPATCHER_OUT");
        ((TDispatcher)p1.getContent()).setContentHandler(p1.getNumPort(),(ContentHandler)p2.getContent());
        break;
      case CONTENT_HANDLER_IN :
        System.out.println("Link un CONTENT_IN not yet implemented !");
        //hum
        //((OneMixer)p1.getContent()).setContentHandler((ContentHandler)p2.getContent());
        break;
      case CONTENT_HANDLER_OUT :
        //System.out.println("Link un CONTENT_HANDLER_OUT");
        if (p1.getContent() instanceof TConnector ) 
          ((TConnector)p1.getContent()).setContentHandler((ContentHandler)p2.getContent());
        else 
          ((TLoopStack1)p1.getContent()).setContentHandler((ContentHandler)p2.getContent());
        break;
      case CONNECTOR_IN :
        System.out.println("Link un CONNECTOR_IN not yet implemented !");
        break;
      case CONNECTOR_OUT :
        System.out.println("Link un CONNECTOR_OUT not yet implemented !");
        break;
      default : 
        System.out.println("Type de port inconnu pour le link : " + p2.getType());
        System.out.println("Type de port inconnu pour le link : " + PortType[p2.getType()]);
        System.out.println("Type de port inconnu pour le link : " + p2.getNumPort());        
        break ;
    } //end switch 
  }
 
  /** fonction to pring Port information */
  public String toString() {
    return "Port :" +iNumPort +" In/out :"+ InOut[iInOut] +" Type ="+ PortType[iTypePort] ;
  } //fin du toString

} //End class

